@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,17 +26,11 @@
 #include "sml_command_internals.h"
 #include "sml_session_internals.h"
 #include "sml_elements_internals.h"
-
-/**
- * @defgroup GroupIDPrivate Group Description Internals
- * @ingroup ParentGroupID
- * @brief The private part
- * 
- */
-/*@{*/
+#include "sml_error_internals.h"
 
 SmlCommandType smlCommandTypeFromString(const char *name, SmlError **error)
 {
+	CHECK_ERROR_REF
 	if (!name)
 		return SML_COMMAND_TYPE_UNKNOWN;
 	
@@ -67,6 +62,7 @@ SmlCommandType smlCommandTypeFromString(const char *name, SmlError **error)
 
 const char *smlCommandTypeToString(SmlCommandType type, SmlError **error)
 {
+	CHECK_ERROR_REF
 	switch (type) {
 		case SML_COMMAND_TYPE_ALERT:
 			return SML_ELEMENT_ALERT;
@@ -96,19 +92,10 @@ const char *smlCommandTypeToString(SmlCommandType type, SmlError **error)
 	return NULL;
 }
 
-/*@}*/
-
-/**
- * @defgroup GroupID Group Description
- * @ingroup ParentGroupID
- * @brief What does this group do?
- * 
- */
-/*@{*/
-
 SmlStatus *smlStatusNew(SmlErrorType data, unsigned int cmdref, unsigned int msgref, SmlLocation *sourceref, SmlLocation *targetref, SmlCommandType type, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%i, %i, %i, %p, %p, %i, %p)", __func__, data, cmdref, msgref, sourceref, targetref, type, error);
+	CHECK_ERROR_REF
 	SmlStatus *status = smlTryMalloc0(sizeof(SmlStatus), error);
 	if (!status)
 		goto error;
@@ -155,7 +142,7 @@ void smlStatusUnref(SmlStatus *status)
 	smlAssert(status);
 	
 	if (g_atomic_int_dec_and_test(&(status->refCount))) {
-		smlTrace(TRACE_INTERNAL, "Refcount == 0!");
+		smlTrace(TRACE_INTERNAL, "%s: Refcount == 0!", __func__);
 		
 		if (status->sourceRef)
 			smlLocationUnref(status->sourceRef);
@@ -163,12 +150,16 @@ void smlStatusUnref(SmlStatus *status)
 		if (status->targetRef)
 			smlLocationUnref(status->targetRef);
 		
-		g_free(status->data);
+		if (status->data)
+			smlSafeCFree(&(status->data));
 		
 		if (status->anchor)
 			smlAnchorFree(status->anchor);
+
+		if (status->item)
+			smlItemUnref(status->item);
 			
-		g_free(status);
+		smlSafeFree((gpointer *)&status);
 	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -203,6 +194,7 @@ SmlBool smlStatusIsResult(SmlStatus *status)
 SmlCommand *smlCommandNew(SmlCommandType type, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%i, %p)", __func__, type, error);
+	CHECK_ERROR_REF
 	
 	SmlCommand *cmd = smlTryMalloc0(sizeof(SmlCommand), error);
 	if (!cmd)
@@ -219,9 +211,10 @@ error:
 	return NULL;
 }
 
-SmlStatus *smlCommandNewReply(SmlCommand *cmd, SmlErrorType code, SmlError **error)
+SmlStatus *smlCommandNewReply(const SmlCommand *cmd, SmlErrorType code, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, cmd, code, error);
+	CHECK_ERROR_REF
 	smlAssert(cmd);
 	
 	SmlStatus *reply = smlStatusNew(code, cmd->cmdID, cmd->msgID, cmd->source, cmd->target, cmd->type, error);
@@ -250,7 +243,8 @@ error:
 
 SmlCommand *smlCommandNewResult(SmlCommand *cmd, SmlLocation *source, char *data, unsigned int size, const char *contenttype, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %i, %s, %p)", __func__, cmd, source, data, size, contenttype, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %i, %s, %p)", __func__, cmd, source, data, size, VA_STRING(contenttype), error);
+	CHECK_ERROR_REF
 	smlAssert(cmd);
 	
 	SmlCommand *result = smlCommandNew(SML_COMMAND_TYPE_RESULTS, error);
@@ -262,6 +256,7 @@ SmlCommand *smlCommandNewResult(SmlCommand *cmd, SmlLocation *source, char *data
 		goto error_free_cmd;
 		
 	result->private.results.status->item = smlItemNewForData(data, size, error);
+	smlSafeCFree(&data);
 	if (!result->private.results.status->item)
 		goto error_free_cmd;
 	
@@ -277,6 +272,8 @@ SmlCommand *smlCommandNewResult(SmlCommand *cmd, SmlLocation *source, char *data
 error_free_cmd:
 	smlCommandUnref(result);
 error:
+	if (data)
+		smlSafeCFree(&data);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;
 }
@@ -298,7 +295,7 @@ void smlCommandUnref(SmlCommand *cmd)
 	smlAssert(cmd);
 	
 	if (g_atomic_int_dec_and_test(&(cmd->refCount))) {
-		smlTrace(TRACE_INTERNAL, "Refcount == 0!");
+		smlTrace(TRACE_INTERNAL, "%s: Refcount == 0!", __func__);
 		
 		if (cmd->parent) {
 			cmd->parent->children = g_list_remove(cmd->parent->children, cmd);
@@ -316,12 +313,12 @@ void smlCommandUnref(SmlCommand *cmd)
 				if (cmd->private.alert.anchor)
 					smlAnchorFree(cmd->private.alert.anchor);
 				if (cmd->private.alert.contentType)
-					g_free(cmd->private.alert.contentType);
+					smlSafeCFree(&(cmd->private.alert.contentType));
 				break;
 			case SML_COMMAND_TYPE_PUT:
 			case SML_COMMAND_TYPE_GET:
 				if (cmd->private.access.type)
-					g_free(cmd->private.access.type);
+					smlSafeCFree(&(cmd->private.access.type));
 					
 				if (cmd->private.access.item)
 					smlItemUnref(cmd->private.access.item);
@@ -329,8 +326,16 @@ void smlCommandUnref(SmlCommand *cmd)
 			case SML_COMMAND_TYPE_ADD:
 			case SML_COMMAND_TYPE_REPLACE:
 			case SML_COMMAND_TYPE_DELETE:
-				if (cmd->private.change.item)
-					smlItemUnref(cmd->private.change.item);
+				if (cmd->private.change.items)
+				{
+					guint i;
+					for (i = 0; i < g_list_length(cmd->private.change.items); i++)
+					{
+						SmlItem *item = g_list_nth_data(cmd->private.change.items, i);
+						smlItemUnref(item);
+					}
+					g_list_free(cmd->private.change.items);
+				}
 				break;
 			case SML_COMMAND_TYPE_MAP:
 				while (cmd->private.map.items) {
@@ -351,7 +356,7 @@ void smlCommandUnref(SmlCommand *cmd)
 		if (cmd->source)
 			smlLocationUnref(cmd->source);
 		
-		g_free(cmd);
+		smlSafeFree((gpointer *)&cmd);
 	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -359,7 +364,8 @@ void smlCommandUnref(SmlCommand *cmd)
 
 SmlCommand *smlCommandNewChange(SmlChangeType type, const char *uid, const char *data, unsigned int size, const char *contenttype, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%i, %s, %p, %i, %s, %p)", __func__, type, uid, data, size, contenttype, error);
+	smlTrace(TRACE_ENTRY, "%s(%i, %s, %p, %i, %s, %p)", __func__, type, VA_STRING(uid), data, size, VA_STRING(contenttype), error);
+	CHECK_ERROR_REF
 	SmlCommand *cmd = NULL;
 	
 	switch (type) {
@@ -377,27 +383,31 @@ SmlCommand *smlCommandNewChange(SmlChangeType type, const char *uid, const char
 	}
 	if (!cmd)
 		goto error;
-	
-	cmd->private.change.item = smlItemNewForData(data, size, error);
-	if (!cmd->private.change.item)
+
+	SmlItem *item = smlItemNewForData(data, size, error);
+	if (!item)
 		goto error_free_cmd;
+	cmd->private.change.items = g_list_append(NULL, item);
+	if (!cmd->private.change.items)
+	{
+		smlItemUnref(item);
+		goto error_free_cmd;
+	}
 	
 	SmlLocation *loc = smlLocationNew(uid, NULL, error);
 	if (!loc)
-		goto error_free_item;
+		goto error_free_cmd;
 	
 	if (type != SML_CHANGE_ADD)
-		cmd->private.change.item->target = loc;
+		item->target = loc;
 	else
-		cmd->private.change.item->source = loc;
+		item->source = loc;
 	
-	cmd->private.change.item->contenttype = g_strdup(contenttype);
+	item->contenttype = g_strdup(contenttype);
 	
 	smlTrace(TRACE_EXIT, "%s: %p", __func__, cmd);
 	return cmd;
 
-error_free_item:
-	smlItemUnref(cmd->private.change.item);
 error_free_cmd:
 	smlCommandUnref(cmd);
 error:
@@ -412,7 +422,8 @@ error:
  */
 SmlCommand *smlCommandNewPartialChange(SmlChangeType type, const char *uid, const char *data, unsigned int complete_size, unsigned int partial_size, const char *contenttype, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%i, %s, %p, %i, %i, %s, %p)", __func__, type, uid, data, complete_size, partial_size, contenttype, error);
+	smlTrace(TRACE_ENTRY, "%s(%i, %s, %p, %i, %i, %s, %p)", __func__, type, VA_STRING(uid), data, complete_size, partial_size, VA_STRING(contenttype), error);
+	CHECK_ERROR_REF
 	SmlCommand *cmd = NULL;
 	
 	switch (type) {
@@ -431,28 +442,32 @@ SmlCommand *smlCommandNewPartialChange(SmlChangeType type, const char *uid, cons
 	if (!cmd)
 		goto error;
 	cmd->size = complete_size;
-	
-	cmd->private.change.item = smlItemNewForData(data, partial_size, error);
-	if (!cmd->private.change.item)
+
+	SmlItem *item = smlItemNewForData(data, partial_size, error);
+	if (!item)
 		goto error_free_cmd;
+	cmd->private.change.items = g_list_append(NULL, item);
+	if (!cmd->private.change.items)
+	{
+		smlItemUnref(item);
+		goto error_free_cmd;
+	}
 	
 	SmlLocation *loc = smlLocationNew(uid, NULL, error);
 	if (!loc)
-		goto error_free_item;
+		goto error_free_cmd;
 	
 	if (type != SML_CHANGE_ADD)
-		cmd->private.change.item->target = loc;
+		item->target = loc;
 	else
-		cmd->private.change.item->source = loc;
+		item->source = loc;
 	
-	cmd->private.change.item->moreData = TRUE;
-	cmd->private.change.item->contenttype = g_strdup(contenttype);
+	item->moreData = TRUE;
+	item->contenttype = g_strdup(contenttype);
 	
 	smlTrace(TRACE_EXIT, "%s: %p", __func__, cmd);
 	return cmd;
 
-error_free_item:
-	smlItemUnref(cmd->private.change.item);
 error_free_cmd:
 	smlCommandUnref(cmd);
 error:
@@ -462,7 +477,8 @@ error:
 
 SmlCommand *smlCommandNewAlert(SmlAlertType type, SmlLocation *target, SmlLocation *source, const char *next, const char *last, const char *contenttype, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%i, %p, %p, %s, %s, %s, %p)", __func__, type, target, source, next, last, contenttype, error);
+	smlTrace(TRACE_ENTRY, "%s(%i, %p, %p, %s, %s, %s, %p)", __func__, type, target, source, VA_STRING(next), VA_STRING(last), VA_STRING(contenttype), error);
+	CHECK_ERROR_REF
 	
 	SmlCommand *cmd = smlCommandNew(SML_COMMAND_TYPE_ALERT, error);
 	if (!cmd)
@@ -477,9 +493,26 @@ SmlCommand *smlCommandNewAlert(SmlAlertType type, SmlLocation *target, SmlLocati
 		cmd->source = source;
 		smlLocationRef(source);
 	}
-	
+
+	/* A synchronization anchor makes only sense if it is not an
+	 * alert for the next part of an incomplete package (next
+	 * message alert) or SAN.
+	 */
 	if (type != SML_ALERT_NEXT_MESSAGE && type != SML_ALERT_TWO_WAY_BY_SERVER) {
-		cmd->private.alert.anchor = smlAnchorNew(last, next, error);
+		/* If this is a slow sync alert then it makes no sense
+		 * to send the last sync anchor because a slow sync is
+		 * only necessary if there was no earlier sync or
+		 * something goes wrong since then which means there is
+		 * no safe last anchor.
+		 */
+		if (type == SML_ALERT_SLOW_SYNC) {
+			if (last)
+				smlTrace(TRACE_INTERNAL,
+					"%s: removing last anchor (%s) from slow sync alert",
+					__func__, VA_STRING(last));
+			cmd->private.alert.anchor = smlAnchorNew(NULL, next, error);
+		} else
+			cmd->private.alert.anchor = smlAnchorNew(last, next, error);
 		if (!cmd->private.alert.anchor)
 			goto error;
 	}
@@ -497,6 +530,7 @@ error:
 SmlCommand *smlCommandNewSync(SmlLocation *target, SmlLocation *source, unsigned int num_changes, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %p)", __func__, target, source, num_changes, error);
+	CHECK_ERROR_REF
 	SmlCommand *cmd = smlCommandNew(SML_COMMAND_TYPE_SYNC, error);
 	if (!cmd)
 		goto error;
@@ -519,7 +553,8 @@ error:
 
 SmlCommand *smlCommandNewPut(SmlLocation *target, SmlLocation *source, const char *data, unsigned int size, const char *contenttype, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %i, %s, %p)", __func__, target, source, data, size, contenttype, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %i, %s, %p)", __func__, target, source, data, size, VA_STRING(contenttype), error);
+	CHECK_ERROR_REF
 	
 	SmlCommand *cmd = smlCommandNew(SML_COMMAND_TYPE_PUT, error);
 	if (!cmd)
@@ -555,7 +590,8 @@ error:
 
 SmlCommand *smlCommandNewGet(SmlLocation *target, const char *contenttype, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, target, contenttype, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, target, VA_STRING(contenttype), error);
+	CHECK_ERROR_REF
 	smlAssert(target);
 	
 	SmlCommand *cmd = smlCommandNew(SML_COMMAND_TYPE_GET, error);
@@ -585,6 +621,7 @@ error:
 SmlCommand *smlCommandNewMap(SmlLocation *target, SmlLocation *source, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, target, source, error);
+	CHECK_ERROR_REF
 	
 	SmlCommand *cmd = smlCommandNew(SML_COMMAND_TYPE_MAP, error);
 	if (!cmd)
@@ -607,6 +644,7 @@ error:
 SmlBool smlCommandAddMapItem(SmlCommand *map, SmlMapItem *item, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, map, item, error);
+	CHECK_ERROR_REF
 	smlAssert(map);
 	smlAssert(map->type == SML_COMMAND_TYPE_MAP);
 	smlAssert(item);
@@ -618,4 +656,24 @@ SmlBool smlCommandAddMapItem(SmlCommand *map, SmlMapItem *item, SmlError **error
 	return TRUE;
 }
 
-/*@}*/
+SmlAlertType smlAlertTypeConvert(unsigned int id, SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%u, %p)", __func__, id, error);
+	CHECK_ERROR_REF
+
+	if (id == 0 ||
+	    id == 100 ||
+	    (200 <= id && id <= 210) ||
+	    (221 <= id && id <= 223))
+	{
+		SmlAlertType result = (SmlAlertType) id;
+		smlTrace(TRACE_EXIT, "%s - %u", __func__, result);
+		return result;
+	} else {
+		smlErrorSet(error, SML_ERROR_GENERIC,
+			"The unknown alert code %u was detected.", id);
+		smlTrace(TRACE_EXIT_ERROR, "%s - %s", __func__, smlErrorPrint(error));
+		return SML_ALERT_UNKNOWN;
+	}	
+}
+
