@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,11 +23,11 @@
 
 #include <libsyncml/sml_session_internals.h>
 
-#include <libsyncml/transports/http_client.h>
-#include <libsyncml/transports/http_server.h>
-
 #define NUM_SESSIONS 30
 
+unsigned int defaultMaxMsgSize = 10240;
+unsigned int defaultMaxObjSize = 1024000;
+
 typedef struct managerTracker {
 	SmlManager *manager;
 	SmlSession *sessions[NUM_SESSIONS];
@@ -41,8 +42,8 @@ START_TEST (manager_new)
 	SmlTransport *server = smlTransportNew(SML_TRANSPORT_HTTP_SERVER, &error);
 	
 	SmlManager *manager = smlManagerNew(server, &error);
-	fail_unless(manager != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(manager != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlManagerFree(manager);
 	
@@ -58,11 +59,11 @@ START_TEST (manager_run)
 	SmlTransport *server = smlTransportNew(SML_TRANSPORT_HTTP_SERVER, &error);
 	
 	SmlManager *manager = smlManagerNew(server, &error);
-	fail_unless(manager != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(manager != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(smlManagerStart(manager, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(manager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlManagerStop(manager);
 	
@@ -72,16 +73,38 @@ START_TEST (manager_run)
 }
 END_TEST
 
-unsigned int transport_errors = 0;
-unsigned int num_sessions = 0;
-unsigned int num_finals = 0;
-unsigned int num_end = 0;
-unsigned int session_errors = 0;
+unsigned int transport_errors;
+unsigned int num_connects;
+unsigned int num_disconnects;
 
-unsigned int num_replies = 0;
-unsigned int num_alerts = 0;
-unsigned int num_syncs = 0;
-unsigned int num_changes = 0;
+unsigned int num_sessions;
+unsigned int num_finals;
+unsigned int num_end;
+unsigned int session_errors;
+
+unsigned int num_replies;
+unsigned int num_alerts;
+unsigned int num_syncs;
+unsigned int num_changes;
+
+void reset_testbed()
+{
+	transport_errors = 0;
+	num_connects = 0;
+	num_disconnects = 0;
+
+	num_sessions = 0;
+	num_finals = 0;
+	num_end = 0;
+	session_errors = 0;
+
+	num_replies = 0;
+	num_alerts = 0;
+	num_syncs = 0;
+	num_changes = 0;
+
+	setup_testbed(NULL);
+}
 
 static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSession *session, SmlError *error, void *userdata)
 {
@@ -94,8 +117,12 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 		case SML_MANAGER_SESSION_FLUSH:
 			break;
 		case SML_MANAGER_CONNECT_DONE:
+			num_connects++;
+			break;
+		case SML_MANAGER_SESSION_ESTABLISHED:
+			break;
 		case SML_MANAGER_DISCONNECT_DONE:
-			fail(NULL);
+			num_disconnects++;
 			break;
 		case SML_MANAGER_TRANSPORT_ERROR:
 			transport_errors++;
@@ -122,36 +149,43 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
+static void _header_callback(SmlSession *session, SmlHeader *header, SmlCred *cred, void *userdata)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, session, header, cred, userdata);
+	smlAssert(session);
+	SmlError *error = NULL;
+
+	SmlStatus *reply = smlStatusNew(SML_AUTH_ACCEPTED, 0, session->lastReceivedMessageID, session->source, session->target, SML_COMMAND_TYPE_HEADER, &error);
+	if (!reply)
+		goto error;
+	
+	if (!smlSessionSendReply(session, reply, &error))
+		goto error;
+	
+	smlStatusUnref(reply);
+
+	session->established = TRUE;
+	
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return;
+
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s - %s", __func__, smlErrorPrint(&error));
+	smlErrorDeref(&error);
+	return;
+}
+
 START_TEST (manager_receive)
 {
-	transport_errors = 0;
-	num_sessions = 0;
-	num_finals = 0;
-	num_end = 0;
-	session_errors = 0;
-	
-	num_replies = 0;
-	num_alerts = 0;
-	num_syncs = 0;
-	num_changes = 0;
-
-	setup_testbed(NULL);
+	reset_testbed();
 	
 	SmlError *error = NULL;
 	SmlTransport *server = smlTransportNew(SML_TRANSPORT_HTTP_SERVER, &error);
 	SmlTransport *client = smlTransportNew(SML_TRANSPORT_HTTP_CLIENT, &error);
-	
-	SmlTransportHttpClientConfig clientConfig;
-	clientConfig.port = 12000;
-	clientConfig.url = "http://127.0.0.1:12000";
-	clientConfig.proxy = NULL;
-	
-	SmlTransportHttpServerConfig serverConfig;
-	serverConfig.port = 12000;
-	serverConfig.url = NULL;
-	serverConfig.interface = NULL;
-	serverConfig.ssl_key = NULL;
-	serverConfig.ssl_crt = NULL;
+
+	sml_fail_unless(smlTransportSetConfigOption(client, "URL", "http://127.0.0.1:12020", &error), NULL);
+
+	sml_fail_unless(smlTransportSetConfigOption(server, "PORT", "12020", &error), NULL);
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -160,35 +194,36 @@ START_TEST (manager_receive)
 	SmlManager *servermanager = servertracker->manager = smlManagerNew(server, &error);
 	smlManagerSetEventCallback(servertracker->manager, _manager_event, servertracker);
 	
-	fail_unless(smlTransportInitialize(client, &clientConfig, &error), NULL);
-	fail_unless(smlTransportInitialize(server, &serverConfig, &error), NULL);
+	sml_fail_unless(smlTransportInitialize(client, &error), "%s", smlErrorPrint(&error));
+	sml_fail_unless(smlTransportInitialize(server, &error), "%s", smlErrorPrint(&error));
 
-	fail_unless(smlManagerStart(clientmanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(smlManagerStart(servermanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(clientmanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(servermanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 
 	char *datastr = "<SyncML></SyncML>";
 	SmlTransportData *data = smlTransportDataNew(datastr, strlen(datastr), SML_MIMETYPE_XML, FALSE, &error);
-	fail_unless(data != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(data != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 
-	fail_unless(smlTransportSend(client, NULL, data, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlTransportSend(client, NULL, data, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlTransportDataDeref(data);
 	
-	while (session_errors != 1) {
+	while (session_errors < 1) {
 		smlManagerDispatch(servermanager);
+		smlManagerDispatch(clientmanager);
 		usleep(100);
 	}
 	
-	fail_unless(transport_errors == 0, NULL);
-	fail_unless(num_sessions == 0, NULL);
-	fail_unless(num_finals == 0, NULL);
-	fail_unless(num_end == 0, NULL);
-	fail_unless(session_errors == 1, NULL);
-	
+	sml_fail_unless(transport_errors == 0, NULL);
+	sml_fail_unless(num_sessions == 0, NULL);
+	sml_fail_unless(num_finals == 0, NULL);
+	sml_fail_unless(num_end == 0, NULL);
+	sml_fail_unless(session_errors == 1, NULL);
+
 	smlManagerStop(clientmanager);
 	smlManagerStop(servermanager);
 	
@@ -197,83 +232,71 @@ START_TEST (manager_receive)
 	
 	g_free(clienttracker);
 	g_free(servertracker);
+
+	sml_fail_unless(smlTransportFinalize(client, &error), NULL);
+	sml_fail_unless(smlTransportFinalize(server, &error), "%s", smlErrorPrint(&error));
 	
-	fail_unless(smlTransportFinalize(server, &error), NULL);
-	fail_unless(smlTransportFinalize(client, &error), NULL);
-	
-	smlTransportFree(server);
 	smlTransportFree(client);
+	smlTransportFree(server);
 }
 END_TEST
 
 START_TEST (manager_receive_session)
 {
-	transport_errors = 0;
-	num_sessions = 0;
-	num_finals = 0;
-	num_end = 0;
-	session_errors = 0;
-	
-	num_replies = 0;
-	num_alerts = 0;
-	num_syncs = 0;
-	num_changes = 0;
-	
-	setup_testbed(NULL);
+	reset_testbed();
 	
 	SmlError *error = NULL;
 	SmlTransport *server = smlTransportNew(SML_TRANSPORT_HTTP_SERVER, &error);
 	SmlTransport *client = smlTransportNew(SML_TRANSPORT_HTTP_CLIENT, &error);
 	
-	SmlTransportHttpClientConfig clientConfig;
-	clientConfig.port = 12001;
-	clientConfig.url = "http://127.0.0.1:12001";
-	clientConfig.proxy = NULL;
-	
-	SmlTransportHttpServerConfig serverConfig;
-	serverConfig.port = 12001;
-	serverConfig.url = NULL;
-	serverConfig.interface = NULL;
-	serverConfig.ssl_key = NULL;
-	serverConfig.ssl_crt = NULL;
-	
+	sml_fail_unless(smlTransportSetConfigOption(client, "URL", "http://127.0.0.1:12001", &error), NULL);
+
+	sml_fail_unless(smlTransportSetConfigOption(server, "PORT", "12001", &error), NULL);
+
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
 	smlManagerSetEventCallback(clienttracker->manager, _manager_event, clienttracker);
 	managerTracker *servertracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *servermanager = servertracker->manager = smlManagerNew(server, &error);
 	smlManagerSetEventCallback(servertracker->manager, _manager_event, servertracker);
+	smlManagerSetLocalMaxMsgSize(servertracker->manager, defaultMaxMsgSize);
+	smlManagerSetLocalMaxObjSize(servertracker->manager, defaultMaxObjSize);
+	smlManagerRegisterHeaderHandler(servermanager, _header_callback, NULL, NULL);
 	
-	fail_unless(smlTransportInitialize(client, &clientConfig, &error), NULL);
-	fail_unless(smlTransportInitialize(server, &serverConfig, &error), NULL);
+	sml_fail_unless(smlTransportInitialize(client, &error), "%s", smlErrorPrint(&error));
+	sml_fail_unless(smlTransportInitialize(server, &error), "%s", smlErrorPrint(&error));
 
-	fail_unless(smlManagerStart(clientmanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(smlManagerStart(servermanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(clientmanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(servermanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 
 	char *datastr = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Alert><CmdID>1</CmdID><Item><Target><LocURI>/test</LocURI></Target><Source><LocURI>/test</LocURI></Source><Meta><Anchor xmlns=\"syncml:metinf\"><Next>Next</Next><Last>last</Last></Anchor></Meta></Item><Data>200</Data></Alert><Final></Final></SyncBody></SyncML>";
 	SmlTransportData *data = smlTransportDataNew(datastr, strlen(datastr), SML_MIMETYPE_XML, FALSE, &error);
-	fail_unless(data != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(data != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 
-	fail_unless(smlTransportSend(client, NULL, data, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlTransportSend(client, NULL, data, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlTransportDataDeref(data);
 	
-	while (num_finals != 1 || session_errors != 1) {
+	/* The server detects that there is no matching datastore and
+	 * client receives an error from the server as the result.
+	 */
+	while (num_finals < 1 || session_errors < 1) {
 		smlManagerDispatch(servermanager);
+		smlManagerDispatch(clientmanager);
 		usleep(100);
 	}
 	
-	fail_unless(transport_errors == 0, NULL);
-	fail_unless(num_sessions == 1, NULL);
-	fail_unless(num_finals == 1, NULL);
-	fail_unless(num_end == 0, NULL);
-	fail_unless(session_errors == 1, NULL);
+	sml_fail_unless(transport_errors == 0, NULL);
+	sml_fail_unless(num_sessions == 1, NULL);
+	sml_fail_unless(num_finals == 1, NULL);
+	sml_fail_unless(num_end == 0, NULL);
+	sml_fail_unless(session_errors == 1, "session errors: %d", session_errors);
 	
-	fail_unless(servertracker->sessions[0] != NULL, NULL);
+	sml_fail_unless(servertracker->sessions[0] != NULL, NULL);
 	smlSessionUnref(servertracker->sessions[0]);
 	
 	g_free(clienttracker);
@@ -285,8 +308,16 @@ START_TEST (manager_receive_session)
 	smlManagerFree(clientmanager);
 	smlManagerFree(servermanager);
 	
-	fail_unless(smlTransportFinalize(server, &error), NULL);
-	fail_unless(smlTransportFinalize(client, &error), NULL);
+	/* The session is not registered correctly at the client
+	 * because HTTP clients create their sessions always by
+	 * themselves and not via automatic registration at the
+	 * manager. Therefore the client transport must be
+	 * explicitely disconnected to avoid warnings from the
+	 * transport layer.
+	 */
+	// sml_fail_unless(smlTransportDisconnect(client, NULL, &error), NULL);
+	sml_fail_unless(smlTransportFinalize(client, &error), NULL);
+	sml_fail_unless(smlTransportFinalize(server, &error), NULL);
 	
 	smlTransportFree(server);
 	smlTransportFree(client);
@@ -295,34 +326,15 @@ END_TEST
 
 START_TEST (manager_end_session)
 {
-	transport_errors = 0;
-	num_sessions = 0;
-	num_finals = 0;
-	num_end = 0;
-	session_errors = 0;
-	
-	num_replies = 0;
-	num_alerts = 0;
-	num_syncs = 0;
-	num_changes = 0;
-	
-	setup_testbed(NULL);
+	reset_testbed();
 	
 	SmlError *error = NULL;
 	SmlTransport *server = smlTransportNew(SML_TRANSPORT_HTTP_SERVER, &error);
 	SmlTransport *client = smlTransportNew(SML_TRANSPORT_HTTP_CLIENT, &error);
 	
-	SmlTransportHttpClientConfig clientConfig;
-	clientConfig.port = 12002;
-	clientConfig.url = "http://127.0.0.1:12002";
-	clientConfig.proxy = NULL;
-	
-	SmlTransportHttpServerConfig serverConfig;
-	serverConfig.port = 12002;
-	serverConfig.url = NULL;
-	serverConfig.interface = NULL;
-	serverConfig.ssl_key = NULL;
-	serverConfig.ssl_crt = NULL;
+	sml_fail_unless(smlTransportSetConfigOption(client, "URL", "http://127.0.0.1:12002", &error), NULL);
+
+	sml_fail_unless(smlTransportSetConfigOption(server, "PORT", "12002", &error), NULL);
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -330,60 +342,112 @@ START_TEST (manager_end_session)
 	managerTracker *servertracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *servermanager = servertracker->manager = smlManagerNew(server, &error);
 	smlManagerSetEventCallback(servertracker->manager, _manager_event, servertracker);
+	smlManagerSetLocalMaxMsgSize(servertracker->manager, defaultMaxMsgSize);
+	smlManagerSetLocalMaxObjSize(servertracker->manager, defaultMaxObjSize);
+	smlManagerRegisterHeaderHandler(servermanager, _header_callback, NULL, NULL);
 	
-	fail_unless(smlTransportInitialize(client, &clientConfig, &error), NULL);
-	fail_unless(smlTransportInitialize(server, &serverConfig, &error), NULL);
+	sml_fail_unless(smlTransportInitialize(client, &error), "%s", smlErrorPrint(&error));
+	sml_fail_unless(smlTransportInitialize(server, &error), "%s", smlErrorPrint(&error));
 
-	fail_unless(smlManagerStart(clientmanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(smlManagerStart(servermanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(clientmanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(servermanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+
+	/* the client send its data to the server */
 
 	char *datastr = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Alert><CmdID>1</CmdID><Item><Target><LocURI>/test</LocURI></Target><Source><LocURI>/test</LocURI></Source><Meta><Anchor xmlns=\"syncml:metinf\"><Next>test1</Next><Last>test2</Last></Anchor></Meta></Item><Data>200</Data></Alert><Final></Final></SyncBody></SyncML>";
 	SmlTransportData *data = smlTransportDataNew(datastr, strlen(datastr), SML_MIMETYPE_XML, FALSE, &error);
-	fail_unless(data != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(data != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 
-	fail_unless(smlTransportSend(client, NULL, data, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlTransportSend(client, NULL, data, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlTransportDataDeref(data);
+
+	/* the server received the data and has no matching datastore */
 	
-	while (num_finals != 1 || session_errors != 1) {
+	while (num_connects < 1 || num_finals < 1 || session_errors < 1) {
 		smlManagerDispatch(servermanager);
 		usleep(100);
 	}
+
+	sml_fail_unless(transport_errors == 0, NULL);
+	sml_fail_unless(num_sessions == 1, NULL);
+	sml_fail_unless(num_finals == 1, NULL);
+	sml_fail_unless(num_end == 0, NULL);
+	sml_fail_unless(session_errors == 1, NULL);
+
+	/* the server sends the answer */
 	
-	fail_unless(transport_errors == 0, NULL);
-	fail_unless(num_sessions == 1, NULL);
-	fail_unless(num_finals == 1, NULL);
-	fail_unless(num_end == 0, NULL);
-	fail_unless(session_errors == 1, NULL);
-	
-	fail_unless(servertracker->sessions[0] != NULL, NULL);
-	fail_unless(smlSessionEnd(servertracker->sessions[0], &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(servertracker->sessions[0] != NULL, NULL);
+	sml_fail_unless(smlSessionEnd(servertracker->sessions[0], &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	smlSessionUnref(servertracker->sessions[0]);
+
+	/* 1. the server sends the error message
+	 * 2. the server declares session end internally
+	 * 3. the server aborts the connection
+	 */
 	
-	while (num_end != 1) {
+	while (num_end < 1) {
 		smlManagerDispatch(servermanager);
 		usleep(100);
 	}
+	sml_fail_unless(TRUE, "server is passive now");
+
+	/* 1. the client receives the connect
+	 * 2. the client receives the error message
+	 *    but the client has no appropriate session
+	 *
+	 * SO WHAT SHOULD HAPPEN !?
+	 *
+	 */
+
+	while (num_connects < 2 || session_errors < 2) {
+		smlManagerDispatch(clientmanager);
+		usleep(100);
+	}
 	
+	sml_fail_unless(transport_errors == 0, NULL);
+	sml_fail_unless(num_sessions == 1, NULL);
+	sml_fail_unless(num_finals == 1, NULL);
+	sml_fail_unless(num_end == 1, NULL);
+	sml_fail_unless(session_errors == 2, "Session errors: %d", session_errors);
+	sml_fail_unless(num_connects == 2, NULL);
+
 	g_free(clienttracker);
 	g_free(servertracker);
 	
+	sml_fail_unless(TRUE, "trackers freed");
+
+	/* first the client must be stopped */
+
+	/* The session is not registered correctly at the client
+	 * because HTTP clients create their sessions always by
+	 * themselves and not via automatic registration at the
+	 * manager. Therefore the client transport must be
+	 * explicitely disconnected to avoid warnings from the
+	 * transport layer.
+	 */
+	sml_fail_unless(smlTransportDisconnect(client, NULL, &error), NULL);
+	while(num_disconnects < 1) {
+		smlManagerDispatch(clientmanager);
+	}
+
 	smlManagerStop(clientmanager);
-	smlManagerStop(servermanager);
-	
 	smlManagerFree(clientmanager);
-	smlManagerFree(servermanager);
-	
-	fail_unless(smlTransportFinalize(server, &error), NULL);
-	fail_unless(smlTransportFinalize(client, &error), NULL);
-	
-	smlTransportFree(server);
+
+	sml_fail_unless(smlTransportFinalize(client, &error), NULL);
 	smlTransportFree(client);
+	
+	/* second the server must be stopped */
+
+	smlManagerStop(servermanager);
+	smlManagerFree(servermanager);
+	sml_fail_unless(smlTransportFinalize(server, &error), NULL);
+	smlTransportFree(server);
 }
 END_TEST
 
@@ -433,58 +497,17 @@ static void _status_reply(SmlSession *session, SmlStatus *status, void *userdata
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _header_callback(SmlSession *session, SmlHeader *header, SmlCred *cred, void *userdata)
-{
-	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, session, header, cred, userdata);
-	smlAssert(session);
-	
-	SmlStatus *reply = smlStatusNew(SML_AUTH_ACCEPTED, 0, session->lastReceivedMessageID, session->source, session->target, SML_COMMAND_TYPE_HEADER, NULL);
-	if (!reply)
-		goto error;
-	
-	if (!smlSessionSendReply(session, reply, NULL))
-		goto error;
-	
-	smlStatusUnref(reply);
-	
-	smlTrace(TRACE_EXIT, "%s", __func__);
-	return;
-
-error:
-	smlTrace(TRACE_EXIT_ERROR, "%s", __func__);
-	return;
-}
-
 START_TEST (manager_start_session)
 {
-	transport_errors = 0;
-	num_sessions = 0;
-	num_finals = 0;
-	num_end = 0;
-	session_errors = 0;
-	
-	num_replies = 0;
-	num_alerts = 0;
-	num_syncs = 0;
-	num_changes = 0;
-	
-	setup_testbed(NULL);
+	reset_testbed();
 	
 	SmlError *error = NULL;
 	SmlTransport *server = smlTransportNew(SML_TRANSPORT_HTTP_SERVER, &error);
 	SmlTransport *client = smlTransportNew(SML_TRANSPORT_HTTP_CLIENT, &error);
 	
-	SmlTransportHttpClientConfig clientConfig;
-	clientConfig.port = 12004;
-	clientConfig.url = "http://127.0.0.1:12004";
-	clientConfig.proxy = NULL;
-	
-	SmlTransportHttpServerConfig serverConfig;
-	serverConfig.port = 12004;
-	serverConfig.url = NULL;
-	serverConfig.interface = NULL;
-	serverConfig.ssl_key = NULL;
-	serverConfig.ssl_crt = NULL;
+	sml_fail_unless(smlTransportSetConfigOption(client, "URL", "http://127.0.0.1:12004", &error), NULL);
+
+	sml_fail_unless(smlTransportSetConfigOption(server, "PORT", "12004", &error), NULL);
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -492,59 +515,66 @@ START_TEST (manager_start_session)
 	managerTracker *servertracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *servermanager = servertracker->manager = smlManagerNew(server, &error);
 	smlManagerSetEventCallback(servertracker->manager, _manager_event, servertracker);
+	smlManagerSetLocalMaxMsgSize(servertracker->manager, defaultMaxMsgSize);
+	smlManagerSetLocalMaxObjSize(servertracker->manager, defaultMaxObjSize);
 	
-	fail_unless(smlTransportInitialize(client, &clientConfig, &error), NULL);
-	fail_unless(smlTransportInitialize(server, &serverConfig, &error), NULL);
+	sml_fail_unless(smlTransportInitialize(client, &error), "%s", smlErrorPrint(&error));
+	sml_fail_unless(smlTransportInitialize(server, &error), "%s", smlErrorPrint(&error));
 
-	fail_unless(smlManagerStart(clientmanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(smlManagerStart(servermanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(clientmanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(servermanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 
 	SmlLocation *loc = smlLocationNew("test", NULL, &error);
-	fail_unless(loc != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(loc != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(smlManagerObjectRegister(servermanager, SML_COMMAND_TYPE_ALERT, NULL, loc, NULL, NULL, _recv_alert, NULL, NULL, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerObjectRegister(servermanager, SML_COMMAND_TYPE_ALERT, NULL, loc, NULL, NULL, _recv_alert, NULL, NULL, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	smlManagerRegisterHeaderHandler(servermanager, _header_callback, NULL, NULL);
 	
 	SmlSession *session = smlSessionNew(SML_SESSION_TYPE_CLIENT, SML_MIMETYPE_XML, SML_VERSION_12, SML_PROTOCOL_SYNCML, loc, loc, NULL, 0, &error);
 	
-	fail_unless(smlManagerSessionAdd(clientmanager, session, NULL, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerSessionAdd(clientmanager, session, NULL, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = smlCommandNewAlert(SML_ALERT_TWO_WAY, loc, loc, "last", "next", NULL, &error);
-	fail_unless(cmd != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(cmd != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlLocationUnref(loc);
 	
-	fail_unless(smlSessionSendCommand(session, cmd, NULL, _status_reply, GINT_TO_POINTER(1), &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlSessionSendCommand(session, cmd, NULL, _status_reply, GINT_TO_POINTER(1), &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlCommandUnref(cmd);
 	
-	fail_unless(smlSessionFlush(session, TRUE, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlSessionFlush(session, TRUE, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
+	/* unref session from smlSessionNew */
 	smlSessionUnref(session);
+	session = NULL;
 	
-	while (num_finals != 1 || num_alerts != 1 || num_sessions != 2) {
+	while (num_finals < 1 || num_alerts < 1 || num_sessions < 2) {
 		smlManagerDispatch(servermanager);
 		smlManagerDispatch(clientmanager);
 		usleep(100);
 	}
 	
-	fail_unless(transport_errors == 0, NULL);
-	fail_unless(num_sessions == 2, NULL);
-	fail_unless(num_finals == 1, NULL);
-	fail_unless(num_end == 0, NULL);
-	fail_unless(session_errors == 0, NULL);
+	sml_fail_unless(transport_errors == 0, NULL);
+	sml_fail_unless(num_sessions == 2, NULL);
+	sml_fail_unless(num_finals == 1, NULL);
+	sml_fail_unless(num_end == 0, NULL);
+	sml_fail_unless(session_errors == 0, NULL);
 	
-	fail_unless(servertracker->sessions[0] != NULL, NULL);
-	fail_unless(smlSessionEnd(servertracker->sessions[0], &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(servertracker->sessions[0] != NULL, NULL);
+	sml_fail_unless(smlSessionEnd(servertracker->sessions[0], &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+
+	/* cleanup references on sessions because of SML_SESSION_NEW event */
+	smlSessionUnref(clienttracker->sessions[0]);
 	smlSessionUnref(servertracker->sessions[0]);
 	
 	while (num_end != 2 || num_replies != 1 || num_finals != 2) {
@@ -554,27 +584,32 @@ START_TEST (manager_start_session)
 	}
 	
 	
-	fail_unless(transport_errors == 0, NULL);
-	fail_unless(num_sessions == 2, NULL);
-	fail_unless(num_finals == 2, NULL);
-	fail_unless(num_end == 2, NULL);
-	fail_unless(session_errors == 0, NULL);
-	fail_unless(num_replies == 1, NULL);
-	fail_unless(num_alerts == 1, NULL);
-	fail_unless(num_syncs == 0, NULL);
-	fail_unless(num_changes == 0, NULL);
+	sml_fail_unless(transport_errors == 0, NULL);
+	sml_fail_unless(num_sessions == 2, NULL);
+	sml_fail_unless(num_finals == 2, NULL);
+	sml_fail_unless(num_end == 2, NULL);
+	sml_fail_unless(session_errors == 0, NULL);
+	sml_fail_unless(num_replies == 1, NULL);
+	sml_fail_unless(num_alerts == 1, NULL);
+	sml_fail_unless(num_syncs == 0, NULL);
+	sml_fail_unless(num_changes == 0, NULL);
 	
 	g_free(clienttracker);
 	g_free(servertracker);
-	
+
 	smlManagerStop(clientmanager);
 	smlManagerStop(servermanager);
-	
+
+	while(num_disconnects < 2) {
+		smlManagerDispatch(clientmanager);
+		smlManagerDispatch(servermanager);
+	}
+
 	smlManagerFree(clientmanager);
 	smlManagerFree(servermanager);
-	
-	fail_unless(smlTransportFinalize(server, &error), NULL);
-	fail_unless(smlTransportFinalize(client, &error), NULL);
+
+	sml_fail_unless(smlTransportFinalize(server, &error), "%s", smlErrorPrint(&error));
+	sml_fail_unless(smlTransportFinalize(client, &error), "%s", smlErrorPrint(&error));
 	
 	smlTransportFree(server);
 	smlTransportFree(client);
@@ -583,35 +618,17 @@ END_TEST
 
 START_TEST (manager_register)
 {
-	transport_errors = 0;
-	num_sessions = 0;
-	num_finals = 0;
-	num_end = 0;
-	session_errors = 0;
+	reset_testbed();
 	
-	num_replies = 0;
-	num_alerts = 0;
-	num_syncs = 0;
-	num_changes = 0;
-	
-	setup_testbed(NULL);
 	SmlError *error = NULL;
 	SmlLocation *loc = smlLocationNew("test", NULL, &error);
 	
 	SmlTransport *server = smlTransportNew(SML_TRANSPORT_HTTP_SERVER, &error);
 	SmlTransport *client = smlTransportNew(SML_TRANSPORT_HTTP_CLIENT, &error);
 	
-	SmlTransportHttpClientConfig clientConfig;
-	clientConfig.port = 12005;
-	clientConfig.url = "http://127.0.0.1:12005";
-	clientConfig.proxy = NULL;
-	
-	SmlTransportHttpServerConfig serverConfig;
-	serverConfig.port = 12005;
-	serverConfig.url = NULL;
-	serverConfig.interface = NULL;
-	serverConfig.ssl_key = NULL;
-	serverConfig.ssl_crt = NULL;
+	sml_fail_unless(smlTransportSetConfigOption(client, "URL", "http://127.0.0.1:12005", &error), NULL);
+
+	sml_fail_unless(smlTransportSetConfigOption(server, "PORT", "12005", &error), NULL);
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -620,16 +637,16 @@ START_TEST (manager_register)
 	SmlManager *servermanager = servertracker->manager = smlManagerNew(server, &error);
 	smlManagerSetEventCallback(servertracker->manager, _manager_event, servertracker);
 	
-	fail_unless(smlTransportInitialize(client, &clientConfig, &error), NULL);
-	fail_unless(smlTransportInitialize(server, &serverConfig, &error), NULL);
+	sml_fail_unless(smlTransportInitialize(client, &error), "%s", smlErrorPrint(&error));
+	sml_fail_unless(smlTransportInitialize(server, &error), "%s", smlErrorPrint(&error));
 
-	fail_unless(smlManagerStart(clientmanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(smlManagerStart(servermanager, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(clientmanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerStart(servermanager, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 
-	fail_unless(smlManagerObjectRegister(clientmanager, SML_COMMAND_TYPE_ALERT, NULL, loc, NULL, NULL, _recv_alert, NULL, NULL, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlManagerObjectRegister(clientmanager, SML_COMMAND_TYPE_ALERT, NULL, loc, NULL, NULL, _recv_alert, NULL, NULL, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	g_free(clienttracker);
 	g_free(servertracker);
@@ -640,8 +657,8 @@ START_TEST (manager_register)
 	smlManagerFree(clientmanager);
 	smlManagerFree(servermanager);
 	
-	fail_unless(smlTransportFinalize(server, &error), NULL);
-	fail_unless(smlTransportFinalize(client, &error), NULL);
+	sml_fail_unless(smlTransportFinalize(server, &error), NULL);
+	sml_fail_unless(smlTransportFinalize(client, &error), NULL);
 	
 	smlLocationUnref(loc);
 	
@@ -653,7 +670,6 @@ END_TEST
 Suite *manager_suite(void)
 {
 	Suite *s = suite_create("Manager");
-	//Suite *s2 = suite_create("Manager");
 	
 	create_case(s, "manager_new", manager_new);
 	create_case(s, "manager_run", manager_run);
@@ -668,14 +684,19 @@ Suite *manager_suite(void)
 
 int main(void)
 {
+	configure_environment();
+
 	int nf;
 
 	Suite *s = manager_suite();
 	
 	SRunner *sr;
 	sr = srunner_create(s);
-	srunner_run_all(sr, CK_NORMAL);
+	srunner_run_all(sr, CK_VERBOSE);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
+
+	cleanup_environment();
+
 	return (nf == 0) ? 0 : 1;
 }
