@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,22 +34,25 @@ static SmlParser *start_wbxml_parser(const char *data)
 	
 	SmlError *error = NULL;
 	SmlParser *parser = smlParserNew(SML_MIMETYPE_WBXML, 0, &error);
-	fail_unless(parser != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	
 #ifdef ENABLE_WBXML
 	char *buffer = NULL;
 	unsigned int size = 0;
-	fail_unless(smlWbxmlConvertTo(NULL, data, &buffer, &size, &error), NULL);
+	sml_fail_unless(smlWbxmlConvertTo(NULL, data, &buffer, &size, &error), NULL);
 
 	if (!smlParserStart(parser, buffer, size, &error)) {
-		fail_unless(error != NULL, NULL);
+		sml_fail_unless(error != NULL, NULL);
+		smlErrorDeref(&error); /* the error is not used */
 		
 		smlParserFree(parser);
+		smlSafeCFree(&buffer);
 		
 		return NULL;
 	}
+	smlSafeCFree(&buffer);
 #endif
 	
 	return parser;
@@ -58,7 +62,7 @@ START_TEST (check_parser_syncml_empty)
 {
 	char *data = "<SyncML></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser == NULL, NULL);
+	sml_fail_unless(parser == NULL, NULL);
 }
 END_TEST
 
@@ -66,20 +70,21 @@ START_TEST (check_parser_no_syncbody)
 {
 	char *data = "<SyncML><SyncHdr></SyncHdr></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header == NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header == NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
-	fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error != NULL, NULL);
+	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
 }
@@ -89,7 +94,7 @@ START_TEST (check_parser_no_synchdr)
 {
 	char *data = "<SyncML><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser == NULL, NULL);
+	sml_fail_unless(parser == NULL, NULL);
 }
 END_TEST
 
@@ -97,20 +102,23 @@ START_TEST (check_parser_synchdr_empty)
 {
 	char *data = "<SyncML><SyncHdr></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header == NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header == NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
-	
-	fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error != NULL, NULL);
+
+	/* The empty SyncHdr is followed by an empty SyncHdr.
+	 * So the parser is at the correct position to finish successfully.
+	 */	
+	sml_fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlParserFree(parser);
 }
@@ -120,20 +128,21 @@ START_TEST (check_parser_synchdr_verproto)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header == NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header == NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
-	fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error != NULL, NULL);
+	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
 }
@@ -143,20 +152,21 @@ START_TEST (check_parser_synchdr_verdtd)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header == NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header == NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
-	fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error != NULL, NULL);
+	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
 }
@@ -166,20 +176,21 @@ START_TEST (check_parser_synchdr_msgid)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header == NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header == NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
-	fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error != NULL, NULL);
+	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
 }
@@ -189,20 +200,21 @@ START_TEST (check_parser_synchdr_sessionid)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header == NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header == NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
-	fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error != NULL, NULL);
+	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
 }
@@ -212,20 +224,21 @@ START_TEST (check_parser_synchdr_target)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header == NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header == NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
-	fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error != NULL, NULL);
+	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
 }
@@ -235,36 +248,37 @@ START_TEST (check_parser_synchdr_source)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
-	
-	fail_unless(header->messageID == 1, NULL);
-	fail_unless(header->target != NULL, NULL);
-	fail_unless(header->source != NULL, NULL);
-	fail_unless(header->protocol == SML_PROTOCOL_SYNCML, NULL);
-	fail_unless(header->version == SML_VERSION_11, NULL);
-	fail_unless(!strcmp(header->sessionID, "1"), NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
+	sml_fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->target != NULL, NULL);
+	sml_fail_unless(header->source != NULL, NULL);
+	sml_fail_unless(header->protocol == SML_PROTOCOL_SYNCML, NULL);
+	sml_fail_unless(header->version == SML_VERSION_11, NULL);
+	sml_fail_unless(!strcmp(header->sessionID, "1"), NULL);
+	smlHeaderFree(header);
+
 	SmlStatus *status = NULL;
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlParserFree(parser);
 }
@@ -274,36 +288,37 @@ START_TEST (check_parser_synchdr_source2)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>asd</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
-	fail_unless(header->target != NULL, NULL);
-	fail_unless(header->source != NULL, NULL);
-	fail_unless(header->protocol == SML_PROTOCOL_SYNCML, NULL);
-	fail_unless(header->version == SML_VERSION_11, NULL);
-	fail_unless(!strcmp(header->sessionID, "asd"), NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->target != NULL, NULL);
+	sml_fail_unless(header->source != NULL, NULL);
+	sml_fail_unless(header->protocol == SML_PROTOCOL_SYNCML, NULL);
+	sml_fail_unless(header->version == SML_VERSION_11, NULL);
+	sml_fail_unless(!strcmp(header->sessionID, "asd"), NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlParserFree(parser);
 }
@@ -313,20 +328,21 @@ START_TEST (check_parser_synchdr_additional)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source><test></test></SyncHdr><SyncBody></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header == NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header == NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
-	fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error != NULL, NULL);
+	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
 }
@@ -336,32 +352,33 @@ START_TEST (check_parser_status_empty)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Status></Status></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(!smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlParserFree(parser);
 }
@@ -371,23 +388,24 @@ START_TEST (check_parser_status_cmdid)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Status><CmdID>1</CmdID></Status></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(!smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 
 	smlParserFree(parser);
@@ -398,23 +416,24 @@ START_TEST (check_parser_status_msgref)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Status><CmdID>1</CmdID><MsgRef>1</MsgRef></Status></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(!smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
@@ -425,23 +444,24 @@ START_TEST (check_parser_status_cmdref)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Status><CmdID>1</CmdID><MsgRef>1</MsgRef><CmdRef>1</CmdRef></Status></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(!smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error != NULL, NULL);
+	sml_fail_unless(!smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error != NULL, NULL);
 	smlErrorDeref(&error);
 	
 	smlParserFree(parser);
@@ -452,35 +472,37 @@ START_TEST (check_parser_status)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Status><CmdID>1</CmdID><MsgRef>1</MsgRef><CmdRef>0</CmdRef><Cmd>Sync</Cmd></Status></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	smlStatusUnref(status);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserEnd(parser, NULL, NULL, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	smlParserFree(parser);
 }
@@ -490,43 +512,46 @@ START_TEST (check_parser_status2)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Status><CmdID>1</CmdID><MsgRef>1</MsgRef><CmdRef>0</CmdRef><Cmd>Sync</Cmd></Status><Status><CmdID>1</CmdID><MsgRef>1</MsgRef><CmdRef>0</CmdRef><Cmd>Sync</Cmd></Status></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	smlStatusUnref(status);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	smlStatusUnref(status);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlBool final = TRUE;
 	SmlBool end = TRUE;
-	fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(final == FALSE, NULL);
-	fail_unless(end == FALSE, NULL);
+	sml_fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(final == FALSE, NULL);
+	sml_fail_unless(end == FALSE, NULL);
 	
 	smlParserFree(parser);
 }
@@ -536,43 +561,46 @@ START_TEST (check_parser_final)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Status><CmdID>1</CmdID><MsgRef>1</MsgRef><CmdRef>0</CmdRef><Cmd>Sync</Cmd></Status><Status><CmdID>1</CmdID><MsgRef>1</MsgRef><CmdRef>0</CmdRef><Cmd>Sync</Cmd></Status><Final></Final></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	smlStatusUnref(status);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	smlStatusUnref(status);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlBool final = FALSE;
 	SmlBool end = FALSE;
-	fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(final == TRUE, NULL);
-	fail_unless(end == TRUE, NULL);
+	sml_fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(final == TRUE, NULL);
+	sml_fail_unless(end == TRUE, NULL);
 	
 	smlParserFree(parser);
 }
@@ -582,43 +610,46 @@ START_TEST (check_parser_final2)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Status><CmdID>1</CmdID><MsgRef>1</MsgRef><CmdRef>0</CmdRef><Cmd>Sync</Cmd></Status><Status><CmdID>1</CmdID><MsgRef>1</MsgRef><CmdRef>0</CmdRef><Cmd>Sync</Cmd></Status><Final/></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	smlStatusUnref(status);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status != NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	smlStatusUnref(status);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlBool final = FALSE;
 	SmlBool end = FALSE;
-	fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(final == TRUE, NULL);
-	fail_unless(end == TRUE, NULL);
+	sml_fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(final == TRUE, NULL);
+	sml_fail_unless(end == TRUE, NULL);
 	
 	smlParserFree(parser);
 }
@@ -628,44 +659,46 @@ START_TEST (check_parser_cmd)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Alert><CmdID>1</CmdID><Item><Target><LocURI>/test</LocURI></Target><Source><LocURI>/test</LocURI></Source><Meta><Anchor xmlns=\"syncml:metinf\"><Next>Next</Next><Last>last</Last></Anchor></Meta></Item><Data>200</Data></Alert></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd != NULL, NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(cmd->cmdID == 1, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(cmd->cmdID == 1, NULL);
+	smlCommandUnref(cmd);
 	
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlBool final = TRUE;
 	SmlBool end = TRUE;
-	fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(final == FALSE, NULL);
-	fail_unless(end == FALSE, NULL);
+	sml_fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(final == FALSE, NULL);
+	sml_fail_unless(end == FALSE, NULL);
 	
 	smlParserFree(parser);
 }
@@ -675,44 +708,46 @@ START_TEST (check_parser_cmd_final)
 {
 	char *data = "<SyncML><SyncHdr><VerProto>SyncML/1.1</VerProto><VerDTD>1.1</VerDTD><MsgID>1</MsgID><SessionID>1</SessionID><Target><LocURI>test</LocURI></Target><Source><LocURI>test</LocURI></Source></SyncHdr><SyncBody><Alert><CmdID>1</CmdID><Item><Target><LocURI>/test</LocURI></Target><Source><LocURI>/test</LocURI></Source><Meta><Anchor xmlns=\"syncml:metinf\"><Next>Next</Next><Last>last</Last></Anchor></Meta></Item><Data>200</Data></Alert><Final></Final></SyncBody></SyncML>";
 	SmlParser *parser = start_wbxml_parser(data);
-	fail_unless(parser != NULL, NULL);
+	sml_fail_unless(parser != NULL, NULL);
 	
 	SmlError *error = NULL;
 	SmlHeader *header = NULL;
 	SmlCred *cred = NULL;
 	
-	fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
-	fail_unless(header != NULL, NULL);
-	fail_unless(cred == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetHeader(parser, &header, &cred, &error), NULL);
+	sml_fail_unless(header != NULL, NULL);
+	sml_fail_unless(cred == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(header->messageID == 1, NULL);
+	sml_fail_unless(header->messageID == 1, NULL);
+	smlHeaderFree(header);
 	
 	SmlStatus *status = NULL;
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
-	fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
-	fail_unless(status == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetStatus(parser, &status, &error), NULL);
+	sml_fail_unless(status == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlCommand *cmd = NULL;
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd != NULL, NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(cmd->cmdID == 1, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd != NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(cmd->cmdID == 1, NULL);
+	smlCommandUnref(cmd);
 	
-	fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
-	fail_unless(cmd == NULL, NULL);
-	fail_unless(error == NULL, NULL);
+	sml_fail_unless(smlParserGetCommand(parser, &cmd, &error), NULL);
+	sml_fail_unless(cmd == NULL, NULL);
+	sml_fail_unless(error == NULL, NULL);
 	
 	SmlBool final = FALSE;
 	SmlBool end = TRUE;
-	fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
-	fail_unless(error == NULL, NULL);
-	fail_unless(final == TRUE, NULL);
-	fail_unless(end == FALSE, NULL);
+	sml_fail_unless(smlParserEnd(parser, &final, &end, &error), NULL);
+	sml_fail_unless(error == NULL, NULL);
+	sml_fail_unless(final == TRUE, NULL);
+	sml_fail_unless(end == FALSE, NULL);
 	
 	smlParserFree(parser);
 }
@@ -751,14 +786,19 @@ Suite *parser_suite(void)
 
 int main(void)
 {
+	configure_environment();
+
 	int nf;
 
 	Suite *s = parser_suite();
 	
 	SRunner *sr;
 	sr = srunner_create(s);
-	srunner_run_all(sr, CK_NORMAL);
+	srunner_run_all(sr, CK_VERBOSE);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
+
+	cleanup_environment();
+
 	return (nf == 0) ? 0 : 1;
 }
